@@ -5,87 +5,28 @@
 # This file is part of cloud-init. See LICENSE file for license information.
 """Red Hat Subscription: Register Red Hat Enterprise Linux based system"""
 
-from logging import Logger
-from textwrap import dedent
+import logging
 
-from cloudinit import log as logging
 from cloudinit import subp, util
 from cloudinit.cloud import Cloud
 from cloudinit.config import Config
-from cloudinit.config.schema import MetaSchema, get_meta_doc
+from cloudinit.config.schema import MetaSchema
 from cloudinit.settings import PER_INSTANCE
 
 LOG = logging.getLogger(__name__)
 
-MODULE_DESCRIPTION = """\
-Register a Red Hat system either by username and password *or* activation and
-org. Following a successful registration, you can:
-
- - auto-attach subscriptions
- - set the service level
- - add subscriptions based on pool id
- - enable/disable yum repositories based on repo id
- - alter the rhsm_baseurl and server-hostname in ``/etc/rhsm/rhs.conf``.
-"""
-
 meta: MetaSchema = {
     "id": "cc_rh_subscription",
-    "name": "Red Hat Subscription",
-    "title": "Register Red Hat Enterprise Linux based system",
-    "description": MODULE_DESCRIPTION,
     "distros": ["fedora", "rhel"],
     "frequency": PER_INSTANCE,
-    "examples": [
-        dedent(
-            """\
-            rh_subscription:
-                username: joe@foo.bar
-                ## Quote your password if it has symbols to be safe
-                password: '1234abcd'
-            """
-        ),
-        dedent(
-            """\
-            rh_subscription:
-                activation-key: foobar
-                org: 12345
-            """
-        ),
-        dedent(
-            """\
-            rh_subscription:
-                activation-key: foobar
-                org: 12345
-                auto-attach: true
-                service-level: self-support
-                add-pool:
-                  - 1a1a1a1a1a1a1a1a1a1a1a1a1a1a1a1a
-                  - 2b2b2b2b2b2b2b2b2b2b2b2b2b2b2b2b
-                enable-repo:
-                  - repo-id-to-enable
-                  - other-repo-id-to-enable
-                disable-repo:
-                  - repo-id-to-disable
-                  - other-repo-id-to-disable
-                # Alter the baseurl in /etc/rhsm/rhsm.conf
-                rhsm-baseurl: http://url
-                # Alter the server hostname in /etc/rhsm/rhsm.conf
-                server-hostname: foo.bar.com
-            """
-        ),
-    ],
     "activate_by_schema_keys": ["rh_subscription"],
 }
 
-__doc__ = get_meta_doc(meta)
 
-
-def handle(
-    name: str, cfg: Config, cloud: Cloud, log: Logger, args: list
-) -> None:
-    sm = SubscriptionManager(cfg, log=log)
+def handle(name: str, cfg: Config, cloud: Cloud, args: list) -> None:
+    sm = SubscriptionManager(cfg)
     if not sm.is_configured():
-        log.debug("%s: module not configured.", name)
+        LOG.debug("%s: module not configured.", name)
         return None
 
     if not sm.is_registered():
@@ -99,21 +40,21 @@ def handle(
                     "Registration failed or did not run completely"
                 )
 
-            # Splitting up the registration, auto-attach, and servicelevel
+            # Splitting up the registration, auto_attach, and servicelevel
             # commands because the error codes, messages from subman are not
             # specific enough.
 
             # Attempt to change the service level
             if sm.auto_attach and sm.servicelevel is not None:
                 if not sm._set_service_level():
-                    raise SubscriptionError("Setting of service-level failed")
+                    raise SubscriptionError("Setting of service_level failed")
                 else:
-                    sm.log.debug("Completed auto-attach with service level")
+                    LOG.debug("Completed auto_attach with service level")
             elif sm.auto_attach:
                 if not sm._set_auto_attach():
-                    raise SubscriptionError("Setting auto-attach failed")
+                    raise SubscriptionError("Setting auto_attach failed")
                 else:
-                    sm.log.debug("Completed auto-attach")
+                    LOG.debug("Completed auto_attach")
 
             if sm.pools is not None:
                 if not isinstance(sm.pools, list):
@@ -128,12 +69,16 @@ def handle(
             return_stat = sm.update_repos()
             if not return_stat:
                 raise SubscriptionError("Unable to add or remove repos")
-            sm.log_success("rh_subscription plugin completed successfully")
+            if sm.release_version:
+                sm._set_release_version()
+                sm._delete_packagemanager_cache()
+
+            LOG.info("rh_subscription plugin completed successfully")
         except SubscriptionError as e:
-            sm.log_warn(str(e))
-            sm.log_warn("rh_subscription plugin did not complete successfully")
+            LOG.warning(str(e))
+            LOG.warning("rh_subscription plugin did not complete successfully")
     else:
-        sm.log_success("System is already registered")
+        LOG.info("System is already registered")
 
 
 class SubscriptionError(Exception):
@@ -141,45 +86,43 @@ class SubscriptionError(Exception):
 
 
 class SubscriptionManager:
-    valid_rh_keys = [
-        "org",
-        "activation-key",
-        "username",
-        "password",
-        "disable-repo",
-        "enable-repo",
-        "add-pool",
-        "rhsm-baseurl",
-        "server-hostname",
-        "auto-attach",
-        "service-level",
-    ]
 
-    def __init__(self, cfg, log=None):
-        if log is None:
-            log = LOG
-        self.log = log
-        self.cfg = cfg
-        self.rhel_cfg = self.cfg.get("rh_subscription", {})
-        self.rhsm_baseurl = self.rhel_cfg.get("rhsm-baseurl")
-        self.server_hostname = self.rhel_cfg.get("server-hostname")
-        self.pools = self.rhel_cfg.get("add-pool")
-        self.activation_key = self.rhel_cfg.get("activation-key")
+    def __init__(self, cfg):
+        self.rhel_cfg = cfg.get("rh_subscription", {})
         self.org = self.rhel_cfg.get("org")
         self.userid = self.rhel_cfg.get("username")
         self.password = self.rhel_cfg.get("password")
-        self.auto_attach = self.rhel_cfg.get("auto-attach")
-        self.enable_repo = self.rhel_cfg.get("enable-repo")
-        self.disable_repo = self.rhel_cfg.get("disable-repo")
-        self.servicelevel = self.rhel_cfg.get("service-level")
+        self.release_version = self.rhel_cfg.get("release_version")
 
-    def log_success(self, msg):
-        """Simple wrapper for logging info messages. Useful for unittests"""
-        self.log.info(msg)
-
-    def log_warn(self, msg):
-        """Simple wrapper for logging warning messages. Useful for unittests"""
-        self.log.warning(msg)
+        # The hyphenated fields have been deprecated in this module.
+        # For new fields in the future, do not use hyphenated fields.
+        # The json schema validator (ran before initializing
+        # SubscriptionManager) already validated that the user did not
+        # pass a deprecated key and its corresponding new key at the same time
+        self.rhsm_baseurl = self.rhel_cfg.get(
+            "rhsm_baseurl", self.rhel_cfg.get("rhsm-baseurl")
+        )
+        self.server_hostname = self.rhel_cfg.get(
+            "server_hostname", self.rhel_cfg.get("server-hostname")
+        )
+        self.pools = self.rhel_cfg.get(
+            "add_pool", self.rhel_cfg.get("add-pool")
+        )
+        self.activation_key = self.rhel_cfg.get(
+            "activation_key", self.rhel_cfg.get("activation-key")
+        )
+        self.auto_attach = self.rhel_cfg.get(
+            "auto_attach", self.rhel_cfg.get("auto-attach")
+        )
+        self.enable_repo = self.rhel_cfg.get(
+            "enable_repo", self.rhel_cfg.get("enable-repo")
+        )
+        self.disable_repo = self.rhel_cfg.get(
+            "disable_repo", self.rhel_cfg.get("disable-repo")
+        )
+        self.servicelevel = self.rhel_cfg.get(
+            "service_level", self.rhel_cfg.get("service-level")
+        )
 
     def _verify_keys(self):
         """
@@ -187,21 +130,12 @@ class SubscriptionManager:
         are what we expect.
         """
 
-        for k in self.rhel_cfg:
-            if k not in self.valid_rh_keys:
-                bad_key = (
-                    "{0} is not a valid key for rh_subscription. "
-                    "Valid keys are: "
-                    "{1}".format(k, ", ".join(self.valid_rh_keys))
-                )
-                return False, bad_key
-
-        # Check for bad auto-attach value
+        # Check for bad auto_attach value
         if (self.auto_attach is not None) and not (
             util.is_true(self.auto_attach) or util.is_false(self.auto_attach)
         ):
             not_bool = (
-                "The key auto-attach must be a boolean value (True/False "
+                "The key auto_attach must be a boolean value (True/False)"
             )
             return False, not_bool
 
@@ -209,11 +143,18 @@ class SubscriptionManager:
             (not self.auto_attach) or (util.is_false(str(self.auto_attach)))
         ):
             no_auto = (
-                "The service-level key must be used in conjunction "
-                "with the auto-attach key.  Please re-run with "
-                "auto-attach: True"
+                "The service_level key must be used in conjunction "
+                "with the auto_attach key.  Please re-run with "
+                "auto_attach: True"
             )
             return False, no_auto
+
+        # Not verifying the release_version statically in _verify_keys
+        # (by verifying the key is in the output of
+        # `subscription-manager release --list`) because sometimes
+        # the release will become available only after enabling some repos
+        # (which is executed after verify_keys). So we will catch this error
+        # during "subscription-manager release --set=<release_version>"
         return True, None
 
     def is_registered(self):
@@ -257,9 +198,7 @@ class SubscriptionManager:
                 return_out = _sub_man_cli(cmd, logstring_val=True)[0]
             except subp.ProcessExecutionError as e:
                 if e.stdout == "":
-                    self.log_warn(
-                        "Registration failed due to: {0}".format(e.stderr)
-                    )
+                    LOG.warning("Registration failed due to: %s", e.stderr)
                 return False
 
         elif (self.userid is not None) and (self.password is not None):
@@ -284,22 +223,20 @@ class SubscriptionManager:
                 return_out = _sub_man_cli(cmd, logstring_val=True)[0]
             except subp.ProcessExecutionError as e:
                 if e.stdout == "":
-                    self.log_warn(
-                        "Registration failed due to: {0}".format(e.stderr)
-                    )
+                    LOG.warning("Registration failed due to: %s", e.stderr)
                 return False
 
         else:
-            self.log_warn(
+            LOG.warning(
                 "Unable to register system due to incomplete information."
             )
-            self.log_warn(
+            LOG.warning(
                 "Use either activationkey and org *or* userid and password"
             )
             return False
 
         reg_id = return_out.split("ID: ")[1].rstrip()
-        self.log.debug("Registered successfully with ID %s", reg_id)
+        LOG.debug("Registered successfully with ID %s", reg_id)
         return True
 
     def _set_service_level(self):
@@ -315,17 +252,16 @@ class SubscriptionManager:
             if e.stdout.rstrip() != "":
                 for line in e.stdout.split("\n"):
                     if line != "":
-                        self.log_warn(line)
+                        LOG.warning(line)
             else:
-                self.log_warn(
-                    "Setting the service level failed with: {0}".format(
-                        e.stderr.strip()
-                    )
+                LOG.warning(
+                    "Setting the service level failed with: %s",
+                    e.stderr.strip(),
                 )
             return False
         for line in return_out.split("\n"):
             if line != "":
-                self.log.debug(line)
+                LOG.debug(line)
         return True
 
     def _set_auto_attach(self):
@@ -333,11 +269,11 @@ class SubscriptionManager:
         try:
             return_out = _sub_man_cli(cmd)[0]
         except subp.ProcessExecutionError as e:
-            self.log_warn("Auto-attach failed with: {0}".format(e))
+            LOG.warning("auto_attach failed with: %s", e)
             return False
         for line in return_out.split("\n"):
             if line != "":
-                self.log.debug(line)
+                LOG.debug(line)
         return True
 
     def _getPools(self):
@@ -389,8 +325,8 @@ class SubscriptionManager:
         """
 
         # An empty list was passed
-        if len(pools) == 0:
-            self.log.debug("No pools to attach")
+        if not pools:
+            LOG.debug("No pools to attach")
             return True
 
         pool_available, pool_consumed = self._getPools()
@@ -400,20 +336,18 @@ class SubscriptionManager:
             if (pool not in pool_consumed) and (pool in pool_available):
                 pool_list.append("--pool={0}".format(pool))
             else:
-                self.log_warn("Pool {0} is not available".format(pool))
+                LOG.warning("Pool %s is not available", pool)
         if len(pool_list) > 0:
             cmd.extend(pool_list)
             try:
                 _sub_man_cli(cmd)
-                self.log.debug(
+                LOG.debug(
                     "Attached the following pools to your system: %s",
                     (", ".join(pool_list)).replace("--pool=", ""),
                 )
                 return True
             except subp.ProcessExecutionError as e:
-                self.log_warn(
-                    "Unable to attach pool {0} due to {1}".format(pool, e)
-                )
+                LOG.warning("Unable to attach pool %s due to %s", pool, e)
                 return False
 
     def update_repos(self):
@@ -430,16 +364,16 @@ class SubscriptionManager:
         if drepos is None:
             drepos = []
         if not isinstance(erepos, list):
-            self.log_warn("Repo IDs must in the format of a list.")
+            LOG.warning("Repo IDs must in the format of a list.")
             return False
 
         if not isinstance(drepos, list):
-            self.log_warn("Repo IDs must in the format of a list.")
+            LOG.warning("Repo IDs must in the format of a list.")
             return False
 
         # Bail if both lists are not populated
-        if (len(erepos) == 0) and (len(drepos) == 0):
-            self.log.debug("No repo IDs to enable or disable")
+        if not (erepos) and not (drepos):
+            LOG.debug("No repo IDs to enable or disable")
             return True
 
         active_repos, inactive_repos = self._getRepos()
@@ -466,14 +400,12 @@ class SubscriptionManager:
             for fail in enable_list_fail:
                 # Check if the repo exists or not
                 if fail in active_repos:
-                    self.log.debug("Repo %s is already enabled", fail)
+                    LOG.debug("Repo %s is already enabled", fail)
                 else:
-                    self.log_warn(
-                        "Repo {0} does not appear to exist".format(fail)
-                    )
+                    LOG.warning("Repo %s does not appear to exist", fail)
         if len(disable_list_fail) > 0:
             for fail in disable_list_fail:
-                self.log.debug(
+                LOG.debug(
                     "Repo %s not disabled because it is not enabled", fail
                 )
 
@@ -487,16 +419,16 @@ class SubscriptionManager:
         try:
             _sub_man_cli(cmd)
         except subp.ProcessExecutionError as e:
-            self.log_warn("Unable to alter repos due to {0}".format(e))
+            LOG.warning("Unable to alter repos due to %s", e)
             return False
 
         if len(enable_list) > 0:
-            self.log.debug(
+            LOG.debug(
                 "Enabled the following repos: %s",
                 (", ".join(enable_list)).replace("--enable=", ""),
             )
         if len(disable_list) > 0:
-            self.log.debug(
+            LOG.debug(
                 "Disabled the following repos: %s",
                 (", ".join(disable_list)).replace("--disable=", ""),
             )
@@ -505,14 +437,39 @@ class SubscriptionManager:
     def is_configured(self):
         return bool((self.userid and self.password) or self.activation_key)
 
+    def _set_release_version(self):
+        """
+        Execute "subscription-manager release --set=<release_version>"
+        Raises Subscription error if the command fails
+        """
+
+        cmd = ["release", f"--set={self.release_version}"]
+        try:
+            _sub_man_cli(cmd)
+        except subp.ProcessExecutionError as e:
+            raise SubscriptionError(
+                f"Unable to set release_version using: {cmd}"
+            ) from e
+
+    def _delete_packagemanager_cache(self):
+        """
+        Delete the package manager cache.
+        Raises Subscription error if the deletion fails
+        """
+        LOG.debug("Deleting the package manager cache")
+        try:
+            util.del_dir("/var/cache/dnf")
+            util.del_dir("/var/cache/yum")
+        except Exception as e:
+            raise SubscriptionError(
+                "Unable to delete the package manager cache"
+            ) from e
+
 
 def _sub_man_cli(cmd, logstring_val=False):
     """
-    Uses the prefered cloud-init subprocess def of subp.subp
+    Uses the preferred cloud-init subprocess def of subp.subp
     and runs subscription-manager.  Breaking this to a
     separate function for later use in mocking and unittests
     """
     return subp.subp(["subscription-manager"] + cmd, logstring=logstring_val)
-
-
-# vi: ts=4 expandtab
