@@ -21,10 +21,10 @@ from cloudinit import url_helper as uhelp
 from cloudinit import util, warnings
 from cloudinit.distros import Distro
 from cloudinit.event import EventScope, EventType
-from cloudinit.net import netplan
+from cloudinit.net import device_driver, netplan
 from cloudinit.net.dhcp import NoDHCPLeaseError
 from cloudinit.net.ephemeral import EphemeralIPNetwork
-from cloudinit.sources import NicOrder
+from cloudinit.sources import HotplugRetrySettings, NicOrder
 from cloudinit.sources.helpers import ec2
 
 LOG = logging.getLogger(__name__)
@@ -34,12 +34,12 @@ STRICT_ID_DEFAULT = "warn"
 
 
 class CloudNames:
-    ALIYUN = "aliyun"
     AWS = "aws"
     BRIGHTBOX = "brightbox"
     ZSTACK = "zstack"
     E24CLOUD = "e24cloud"
     OUTSCALE = "outscale"
+    TILAA = "tilaa"
     # UNKNOWN indicates no positive id.  If strict_id is 'warn' or 'false',
     # then an attempt at the Ec2 Metadata service will be made.
     UNKNOWN = "unknown"
@@ -54,7 +54,7 @@ def skip_404_tag_errors(exception):
 
 
 # Cloud platforms that support IMDSv2 style metadata server
-IDMSV2_SUPPORTED_CLOUD_PLATFORMS = [CloudNames.AWS, CloudNames.ALIYUN]
+IDMSV2_SUPPORTED_CLOUD_PLATFORMS = [CloudNames.AWS]
 
 # Only trigger hook-hotplug on NICs with Ec2 drivers. Avoid triggering
 # it on docker virtual NICs and the like. LP: #1946003
@@ -63,6 +63,11 @@ ENV{ID_NET_DRIVER}=="vif|ena|ixgbevf", GOTO="cloudinit_hook"
 GOTO="cloudinit_end"
 """
 
+# Drivers that indicate a NIC is being provided by EC2
+# as an Elastic Network Adaptor or Elastic Fabric Adapter
+# https://github.com/amzn/amzn-drivers/
+ELASTIC_DRIVERS = ["ena", "efa"]
+
 
 class DataSourceEc2(sources.DataSource):
     dsname = "Ec2"
@@ -72,7 +77,6 @@ class DataSourceEc2(sources.DataSource):
     metadata_urls = [
         "http://169.254.169.254",
         "http://[fd00:ec2::254]",
-        "http://instance-data.:8773",
     ]
 
     # The minimum supported metadata_version from the ec2 metadata apis
@@ -114,6 +118,7 @@ class DataSourceEc2(sources.DataSource):
     }
 
     extra_hotplug_udev_rules = _EXTRA_HOTPLUG_UDEV_RULES
+    hotplug_retry_settings = HotplugRetrySettings(True, 5, 30)
 
     def __init__(self, sys_cfg, distro, paths):
         super(DataSourceEc2, self).__init__(sys_cfg, distro, paths)
@@ -125,6 +130,7 @@ class DataSourceEc2(sources.DataSource):
         super()._unpickle(ci_pkl_version)
         self.extra_hotplug_udev_rules = _EXTRA_HOTPLUG_UDEV_RULES
         self._fallback_nic_order = NicOrder.MAC
+        self.hotplug_retry_settings = HotplugRetrySettings(True, 5, 30)
 
     def _get_cloud_name(self):
         """Return the cloud name as identified during _get_data."""
@@ -153,24 +159,38 @@ class DataSourceEc2(sources.DataSource):
             if util.is_FreeBSD():
                 LOG.debug("FreeBSD doesn't support running dhclient with -sf")
                 return False
-            try:
-                with EphemeralIPNetwork(
-                    self.distro,
-                    self.distro.fallback_interface,
-                    ipv4=True,
-                    ipv6=True,
-                ) as netw:
-                    self._crawled_metadata = self.crawl_metadata()
-                    LOG.debug(
-                        "Crawled metadata service%s",
-                        f" {netw.state_msg}" if netw.state_msg else "",
-                    )
-
-            except NoDHCPLeaseError:
+            candidate_nics = net.find_candidate_nics()
+            LOG.debug("Looking for the primary NIC in: %s", candidate_nics)
+            if len(candidate_nics) < 1:
+                LOG.error("The instance must have at least one eligible NIC")
                 return False
+            for candidate_nic in sorted(
+                candidate_nics, key=_prefer_elastic_drivers
+            ):
+                try:
+                    with EphemeralIPNetwork(
+                        self.distro,
+                        candidate_nic,
+                        ipv4=True,
+                        ipv6=True,
+                    ) as netw:
+                        self._crawled_metadata = self.crawl_metadata()
+                        if self._crawled_metadata:
+                            self.distro.fallback_interface = candidate_nic
+                            LOG.debug("Set fallback NIC: %s.", candidate_nic)
+                            LOG.debug(
+                                "Crawled metadata service%s",
+                                f" {netw.state_msg}" if netw.state_msg else "",
+                            )
+                            break
+                except NoDHCPLeaseError:
+                    LOG.debug(
+                        "Unable to obtain a DHCP lease for %s", candidate_nic
+                    )
         else:
             self._crawled_metadata = self.crawl_metadata()
         if not self._crawled_metadata:
+            LOG.error("Unable to get metadata")
             return False
         self.metadata = self._crawled_metadata.get("meta-data", None)
         self.userdata_raw = self._crawled_metadata.get("user-data", None)
@@ -775,11 +795,6 @@ def warn_if_necessary(cfgval, cfg):
     warnings.show_warning("non_ec2_md", cfg, mode=True, sleep=sleep)
 
 
-def identify_aliyun(data):
-    if data["product_name"] == "Alibaba Cloud ECS":
-        return CloudNames.ALIYUN
-
-
 def identify_aws(data):
     # data is a dictionary returned by _collect_platform_data.
     uuid_str = data["uuid"]
@@ -806,6 +821,11 @@ def identify_zstack(data):
         return CloudNames.ZSTACK
 
 
+def identify_tilaa(data):
+    if data["vendor"] == "Tilaa":
+        return CloudNames.TILAA
+
+
 def identify_e24cloud(data):
     if data["vendor"] == "e24cloud":
         return CloudNames.E24CLOUD
@@ -828,7 +848,7 @@ def identify_platform():
         identify_zstack,
         identify_e24cloud,
         identify_outscale,
-        identify_aliyun,
+        identify_tilaa,
         lambda x: CloudNames.UNKNOWN,
     )
     for checker in checks:
@@ -874,6 +894,22 @@ def _collect_platform_data():
     }
 
 
+def _prefer_elastic_drivers(nic: str) -> int:
+    """Sorts the NICs so that Amazon drivers are first.
+
+    This helps speed up finding the metadata server since it will generally
+    be reachable via the first ENA/EFA NIC if one is present. Each incorrect
+    NIC that we are able to skip shortens boot by approximately
+    DataSourceEc2.url_max_wait seconds.
+    """
+    # The python builtin `sorted` is guaranteed to be stable,
+    # so we only need to sort
+    # based on whether the NIC is an elastic driver or not
+    if device_driver(nic) in ELASTIC_DRIVERS:
+        return 0
+    return 1
+
+
 def _build_nic_order(
     macs_metadata: Dict[str, Dict],
     macs_to_nics: Dict[str, str],
@@ -893,7 +929,7 @@ def _build_nic_order(
     @return: Dictionary with macs as keys and nic orders as values.
     """
     nic_order: Dict[str, int] = {}
-    if len(macs_to_nics) == 0 or len(macs_metadata) == 0:
+    if (not macs_to_nics) or (not macs_metadata):
         return nic_order
 
     valid_macs_metadata = filter(
