@@ -2,12 +2,15 @@ import pytest
 
 from tests.integration_tests.instances import IntegrationInstance
 from tests.integration_tests.integration_settings import PLATFORM
-from tests.integration_tests.releases import CURRENT_RELEASE
+from tests.integration_tests.releases import CURRENT_RELEASE, JAMMY
 
 USER_DATA = """\
 #cloud-config
 runcmd:
  - echo "hi" >> /var/tmp/hi
+snap:
+ commands:
+   00: echo heya
 """
 
 
@@ -21,15 +24,22 @@ def test_frequency_override(client: IntegrationInstance):
     assert client.read_from_file("/var/tmp/hi").strip().count("hi") == 1
     # This workaround is not needed for OCI, so just skip it
     if CURRENT_RELEASE.os == "ubuntu" and PLATFORM != "oci":
-        if CURRENT_RELEASE.series in ("focal", "jammy", "lunar", "mantic"):
-            # Stable series will block on snapd.seeded.service and create a
-            # semaphore file
-            assert client.execute("test -f /var/lib/cloud/snap-seeded.once").ok
+        if CURRENT_RELEASE > JAMMY:
+            # Newer series will not block on snapd.seeded.service
+            assert "snapd.seeded.service" not in client.execute(
+                "systemctl show -P After cloud-config.service"
+            )
+            assert client.execute(
+                "test -f /var/lib/cloud/sem/snap_seeded.once"
+            ).ok
         else:
-            # Newer series will not block on snapd.seeded.service nor create a
-            # semaphore file
+            # Jammy series and earlier will block on snapd.seeded.service
+            assert "snapd.seeded.service" in client.execute(
+                "systemctl show -P After cloud-config.service"
+            )
+            # Jammy will not create a snap-seeded semaphore.
             assert not client.execute(
-                "test -f /var/lib/cloud/snap-seeded.once"
+                "test -f /var/lib/cloud/sem/snap_seeded.once"
             ).ok
 
     # Change frequency of scripts_user to always
