@@ -11,79 +11,85 @@ from cloudinit.config.schema import (
     get_schema,
     validate_cloudconfig_schema,
 )
-from tests.unittests import helpers as t_help
 from tests.unittests.helpers import mock, skipUnlessJsonSchema
 
 
-class TestLoadPowerState(t_help.TestCase):
-    def setUp(self):
-        super(TestLoadPowerState, self).setUp()
-        cls = distros.fetch("ubuntu")
-        paths = helpers.Paths({})
-        self.dist = cls("ubuntu", {}, paths)
+@pytest.fixture
+def dist():
+    cls = distros.fetch("ubuntu")
+    paths = helpers.Paths({})
+    return cls("ubuntu", {}, paths)
 
-    def test_no_config(self):
+
+class TestLoadPowerState:
+
+    def test_no_config(self, dist):
         # completely empty config should mean do nothing
-        (cmd, _timeout, _condition) = psc.load_power_state({}, self.dist)
-        self.assertIsNone(cmd)
+        (cmd, _timeout, _condition) = psc.load_power_state({}, dist)
+        assert cmd is None
 
-    def test_irrelevant_config(self):
+    def test_irrelevant_config(self, dist):
         # no power_state field in config should return None for cmd
         (cmd, _timeout, _condition) = psc.load_power_state(
-            {"foo": "bar"}, self.dist
+            {"foo": "bar"}, dist
         )
-        self.assertIsNone(cmd)
+        assert cmd is None
 
-    def test_invalid_mode(self):
+    def test_invalid_mode(self, dist):
 
         cfg = {"power_state": {"mode": "gibberish"}}
-        self.assertRaises(TypeError, psc.load_power_state, cfg, self.dist)
+        with pytest.raises(TypeError):
+            psc.load_power_state(cfg, dist)
 
         cfg = {"power_state": {"mode": ""}}
-        self.assertRaises(TypeError, psc.load_power_state, cfg, self.dist)
+        with pytest.raises(TypeError):
+            psc.load_power_state(cfg, dist)
 
-    def test_empty_mode(self):
+    def test_empty_mode(self, dist):
         cfg = {"power_state": {"message": "goodbye"}}
-        self.assertRaises(TypeError, psc.load_power_state, cfg, self.dist)
+        with pytest.raises(TypeError):
+            psc.load_power_state(cfg, dist)
 
-    def test_valid_modes(self):
-        cfg = {"power_state": {}}
+    def test_valid_modes(self, dist):
+        cfg: dict = {"power_state": {}}
         for mode in ("halt", "poweroff", "reboot"):
             cfg["power_state"]["mode"] = mode
-            check_lps_ret(psc.load_power_state(cfg, self.dist), mode=mode)
+            check_lps_ret(psc.load_power_state(cfg, dist), mode=mode)
 
-    def test_invalid_delay(self):
+    def test_invalid_delay(self, dist):
         cfg = {"power_state": {"mode": "poweroff", "delay": "goodbye"}}
-        self.assertRaises(TypeError, psc.load_power_state, cfg, self.dist)
+        with pytest.raises(TypeError):
+            psc.load_power_state(cfg, dist)
 
-    def test_valid_delay(self):
+    def test_valid_delay(self, dist):
         cfg = {"power_state": {"mode": "poweroff", "delay": ""}}
         for delay in ("now", "+1", "+30"):
             cfg["power_state"]["delay"] = delay
-            check_lps_ret(psc.load_power_state(cfg, self.dist))
+            check_lps_ret(psc.load_power_state(cfg, dist))
 
-    def test_message_present(self):
+    def test_message_present(self, dist):
         cfg = {"power_state": {"mode": "poweroff", "message": "GOODBYE"}}
-        ret = psc.load_power_state(cfg, self.dist)
-        check_lps_ret(psc.load_power_state(cfg, self.dist))
-        self.assertIn(cfg["power_state"]["message"], ret[0])
+        ret = psc.load_power_state(cfg, dist)
+        check_lps_ret(ret)
+        assert cfg["power_state"]["message"] in ret[0]
 
-    def test_no_message(self):
+    def test_no_message(self, dist):
         # if message is not present, then no argument should be passed for it
         cfg = {"power_state": {"mode": "poweroff"}}
-        (cmd, _timeout, _condition) = psc.load_power_state(cfg, self.dist)
-        self.assertNotIn("", cmd)
-        check_lps_ret(psc.load_power_state(cfg, self.dist))
-        self.assertTrue(len(cmd) == 3)
+        (cmd, _timeout, _condition) = psc.load_power_state(cfg, dist)
+        assert "" not in cmd
+        check_lps_ret(psc.load_power_state(cfg, dist))
+        assert len(cmd) == 3
 
-    def test_condition_null_raises(self):
+    def test_condition_null_raises(self, dist):
         cfg = {"power_state": {"mode": "poweroff", "condition": None}}
-        self.assertRaises(TypeError, psc.load_power_state, cfg, self.dist)
+        with pytest.raises(TypeError):
+            psc.load_power_state(cfg, dist)
 
-    def test_condition_default_is_true(self):
+    def test_condition_default_is_true(self, dist):
         cfg = {"power_state": {"mode": "poweroff"}}
-        _cmd, _timeout, cond = psc.load_power_state(cfg, self.dist)
-        self.assertEqual(cond, True)
+        _cmd, _timeout, cond = psc.load_power_state(cfg, dist)
+        assert cond is True
 
     def test_freebsd_poweroff_uses_lowercase_p(self):
         with mock.patch(
@@ -95,7 +101,7 @@ class TestLoadPowerState(t_help.TestCase):
             freebsd = cls("freebsd", {}, paths)
             cfg = {"power_state": {"mode": "poweroff"}}
             ret = psc.load_power_state(cfg, freebsd)
-            self.assertIn("-p", ret[0])
+            assert "-p" in ret[0]
 
     def test_alpine_delay(self):
         # alpine takes delay in seconds.
@@ -106,30 +112,30 @@ class TestLoadPowerState(t_help.TestCase):
         for delay, value in (("now", 0), ("+1", 60), ("+30", 1800)):
             cfg["power_state"]["delay"] = delay
             ret = psc.load_power_state(cfg, alpine)
-            self.assertEqual("-d", ret[0][1])
-            self.assertEqual(str(value), ret[0][2])
+            assert ret[0][1] == "-d"
+            assert ret[0][2] == str(value)
 
 
-class TestCheckCondition(t_help.TestCase):
+class TestCheckCondition:
     def cmd_with_exit(self, rc):
         return [sys.executable, "-c", "import sys; sys.exit(%s)" % rc]
 
     def test_true_is_true(self):
-        self.assertEqual(psc.check_condition(True), True)
+        assert psc.check_condition(True) is True
 
     def test_false_is_false(self):
-        self.assertEqual(psc.check_condition(False), False)
+        assert psc.check_condition(False) is False
 
     def test_cmd_exit_zero_true(self):
-        self.assertEqual(psc.check_condition(self.cmd_with_exit(0)), True)
+        assert psc.check_condition(self.cmd_with_exit(0)) is True
 
     def test_cmd_exit_one_false(self):
-        self.assertEqual(psc.check_condition(self.cmd_with_exit(1)), False)
+        assert psc.check_condition(self.cmd_with_exit(1)) is False
 
     @mock.patch("cloudinit.config.cc_power_state_change.LOG")
     def test_cmd_exit_nonzero_warns(self, mocklog):
-        self.assertEqual(psc.check_condition(self.cmd_with_exit(2)), False)
-        self.assertEqual(mocklog.warning.call_count, 1)
+        assert psc.check_condition(self.cmd_with_exit(2)) is False
+        assert mocklog.warning.call_count == 1
 
 
 def check_lps_ret(psc_return, mode=None):
@@ -213,5 +219,7 @@ class TestPowerStateChangeSchema:
         if error_msg is None:
             validate_cloudconfig_schema(config, get_schema(), strict=True)
         else:
-            with pytest.raises(SchemaValidationError, match=error_msg):
+            with pytest.raises(
+                SchemaValidationError, match=error_msg if error_msg else None
+            ):
                 validate_cloudconfig_schema(config, get_schema(), strict=True)
